==> create-sdk-1.8.3.tgz: package/src/utils.ts <==
import { DenoBridge } from "@netlify/edge-bundler";
import { join, relative, dirname } from "path";
import { readFile, stat, mkdir, writeFile } from "fs/promises";
import * as JSONC from "comment-json";
import { confirm, rawlist } from "@inquirer/prompts";
import unixify from "unixify";
import open from "open";
import { BOILERPLATE_OPTIONS } from "./constants.js";
import { execSync } from "child_process";

export const PACKAGE_MANAGERS = ["npm", "yarn", "pnpm"] as const;
export type PackageManager = (typeof PACKAGE_MANAGERS)[number];

export const isPackageManager = (value: string): value is PackageManager => {
  return PACKAGE_MANAGERS.includes(value as PackageManager);
};

const PACKAGE_MANAGER_COMMANDS = {
  install: {
    npm: "npm install",
    yarn: "yarn",
    pnpm: "pnpm install",
  },
  add: {
    npm: "npm install",
    yarn: "yarn add",
    pnpm: "pnpm add",
  },
  run: {
    npm: "npx",
    yarn: "yarn",
    pnpm: "pnpm",
  },
} as const;

export const getCommand = (
  command: keyof typeof PACKAGE_MANAGER_COMMANDS,
  packageManager: PackageManager,
  args?: string[],
) => {
  const pkgManagerCommand = PACKAGE_MANAGER_COMMANDS[command][packageManager];

  if (!args) {
    return pkgManagerCommand;
  }

  return `${pkgManagerCommand} ${args.join(" ")}`;
};

export const getSettings = async (settingsPath: string) => {
  try {
    const stats = await stat(settingsPath);

    if (!stats.isFile()) {
      throw new Error(`${settingsPath} is not a valid file.`);
    }

    const file = await readFile(settingsPath, "utf8");

    return {
      fileExists: true,
      settings: JSONC.parse(file),
    };
  } catch (error) {
    if ((error as NodeJS.ErrnoException).code !== "ENOENT") {
      throw new Error(
        `Could not open VS Code settings file: ${
          (error as NodeJS.ErrnoException).message
        }`,
      );
    }

    return {
      fileExists: false,
      settings: {},
    };
  }
};

export const applySettings = (
  existingSettings: any,
  {
    denoBinary,
    edgeFunctionsPath,
    destination,
  }: {
    denoBinary: { global: boolean; path: string };
    edgeFunctionsPath: string;
    destination: string;
  },
) => {
  const relativeEdgeFunctionsPath = unixify(
    relative(destination, edgeFunctionsPath),
  );
  const settings = JSONC.assign(existingSettings, {
    "deno.enable": true,
    "deno.enablePaths": existingSettings["deno.enablePaths"] || [],
    "deno.unstable": true,
  });

  // If the Edge Functions path isn't already in `deno.enabledPaths`, let's add
  // it.
  if (!settings["deno.enablePaths"].includes(relativeEdgeFunctionsPath)) {
    settings["deno.enablePaths"].push(relativeEdgeFunctionsPath);
  }

  // If the Deno CLI binary isn't globally installed, we need to set the path
  // to it in the settings file or the extension won't know where to find it.
  // The only exception is when `deno.path` has already been defined, because
  // we don't want to override that.
  if (!denoBinary.global && settings["deno.path"] === undefined) {
    settings["deno.path"] = denoBinary.path;
  }

  return settings;
};

export const writeSettings = async ({
  settings,
  settingsPath,
}: {
  settings: any;
  settingsPath: string;
}) => {
  const serializedSettings = JSONC.stringify(settings, null, 2);

  await mkdir(dirname(settingsPath), { recursive: true });
  await writeFile(settingsPath, serializedSettings);
};

export const promptEditorHelper = async ({
  destination,
}: {
  destination: string;
}) => {
  const deno = new DenoBridge({
    onBeforeDownload: () =>
      console.log(
        `Setting up the Edge Functions environment. This may take a couple of minutes.`,
      ),
  });
  const denoBinary = await deno.getBinaryPath();
  const settingsPath = join(destination, ".vscode", "settings.json");
  const edgeFunctionsPath = join(destination, "src", "edge-functions");
  const { fileExists, settings: existingSettings } =
    await getSettings(settingsPath);
  const settings = applySettings(existingSettings, {
    denoBinary,
    edgeFunctionsPath,
    destination,
  });

  const createOrUpdateSettings = await confirm({
    message: fileExists
      ? `There is a VS Code settings file at ${settingsPath}. Can we update it?`
      : `A new VS Code settings file will be created at ${settingsPath}`,
    default: true,
  });

  if (!createOrUpdateSettings) {
    return;
  }

  try {
    const denoExtConfirm = await confirm({
      message:
        "The Deno VS Code extension is recommended. Would you like to install it now?",
      default: true,
    });

    if (denoExtConfirm) {
      console.log("Opening your browser...");
      open("https://ntl.fyi/deno-vscode");
    }
  } catch {
    console.log(
      "Something went wrong opening the browser, install the deno vs-code extension manually here: https://ntl.fyi/deno-vscode",
    );
  }

  try {
    await writeSettings({ settings, settingsPath });

    console.log(`VS Code settings file ${fileExists ? "updated" : "created"}.`);
  } catch {
    throw new Error("Could not write VS Code settings file.");
  }
};

const askIntegrationLevel = async (boilerplate: string[]) => {
  return await rawlist({
    message: `What level will this integration be installed on?`,
    choices: [
      {
        name:
          boilerplate.includes(BOILERPLATE_OPTIONS.build) ||
          boilerplate.includes(BOILERPLATE_OPTIONS.edgeFunctions) ||
          boilerplate.includes(BOILERPLATE_OPTIONS.functions)
            ? "site (recommended for Build Event Handlers, Edge Functions and Functions)"
            : "site",
        value: "site",
      },
      {
        name: boilerplate.includes(BOILERPLATE_OPTIONS.connector)
          ? "team (required for Connectors)"
          : "team",
        value: "team",
      },
      { name: "team-and-site", value: "team-and-site" },
    ],
  });
};

export const decideIntegrationLevel = async (boilerplate: string[]) => {
  if (boilerplate.length === 1) {
    if (boilerplate[0] === BOILERPLATE_OPTIONS.build) {
      return "site";
    } else if (boilerplate[0] === BOILERPLATE_OPTIONS.connector) {
      return "team";
    } else {
      return await askIntegrationLevel(boilerplate);
    }
  } else {
    if (
      boilerplate.includes(BOILERPLATE_OPTIONS.build) &&
      boilerplate.includes(BOILERPLATE_OPTIONS.connector)
    ) {
      return "team-and-site";
    } else {
      return await askIntegrationLevel(boilerplate);
    }
  }
};

export const isNetlifyCLIInstalled = () => {
  try {
    execSync("netlify --version", { stdio: "ignore" });
    return true;
  } catch {
    return false;
  }
};

==> create-sdk-2.4.0-pre.84b8417.tgz: package/src/utils.ts <==
import { DenoBridge } from "@netlify/edge-bundler";
import { join, relative, dirname } from "path";
import { readFile, stat, mkdir, writeFile } from "fs/promises";
import * as JSONC from "comment-json";
import { confirm, select } from "@inquirer/prompts";
import unixify from "unixify";
import open from "open";
import { execSync } from "child_process";
import chalk from "chalk";
import semver from "semver";

export const PACKAGE_MANAGERS = ["npm", "yarn", "pnpm"] as const;
export type PackageManager = (typeof PACKAGE_MANAGERS)[number];

export const isPackageManager = (value: string): value is PackageManager => {
  return PACKAGE_MANAGERS.includes(value as PackageManager);
};

// The range of officially supported Node versions.
// If you update this value, please update the supportedNodeVersionRange in packages/sdk/src/bin/index.ts to match.
const supportedNodeVersionRange = ">= 18.19.0";
const ignoreSupportedVersionError =
  process.env.NETLIFY_SDK_IGNORE_SUPPORTED_NODE_VERSION_RANGE === "true";

export function checkNodeVersion() {
  if (!semver.satisfies(process.versions.node, supportedNodeVersionRange)) {
    console.error(
      chalk.red(
        `
ERROR: The Netlify SDK does not support the current Node version: ${process.versions.node}.
       The range of officially supported Node versions is ${supportedNodeVersionRange}.

       Please install a supported version of node and re-run the process.
      `.trim(),
      ),
    );
    if (!ignoreSupportedVersionError) {
      process.exit(1);
    }
  }
}

export const exitWithError = (message: string) => {
  console.error(message);
  process.exit(0);
};

const PACKAGE_MANAGER_COMMANDS = {
  install: {
    npm: "npm install",
    yarn: "yarn",
    pnpm: "pnpm install",
  },
  add: {
    npm: "npm install",
    yarn: "yarn add",
    pnpm: "pnpm add",
  },
  run: {
    npm: "npx",
    yarn: "yarn",
    pnpm: "pnpm",
  },
} as const;

export const getCommand = (
  command: keyof typeof PACKAGE_MANAGER_COMMANDS,
  packageManager: PackageManager,
  args?: string[],
) => {
  const pkgManagerCommand = PACKAGE_MANAGER_COMMANDS[command][packageManager];

  if (!args) {
    return pkgManagerCommand;
  }

  return `${pkgManagerCommand} ${args.join(" ")}`;
};

export const getSettings = async (settingsPath: string) => {
  try {
    const stats = await stat(settingsPath);

    if (!stats.isFile()) {
      exitWithError(`${settingsPath} is not a valid file.`);
    }

    const file = await readFile(settingsPath, "utf8");

    return {
      fileExists: true,
      settings: JSONC.parse(file),
    };
  } catch (error) {
    if ((error as NodeJS.ErrnoException).code !== "ENOENT") {
      exitWithError(
        `Could not open VS Code settings file: ${
          (error as NodeJS.ErrnoException).message
        }`,
      );
    }

    return {
      fileExists: false,
      settings: {},
    };
  }
};

export const applySettings = (
  existingSettings: any,
  {
    denoBinary,
    edgeFunctionsPath,
    destination,
  }: {
    denoBinary: { global: boolean; path: string };
    edgeFunctionsPath: string;
    destination: string;
  },
) => {
  const relativeEdgeFunctionsPath = unixify(
    relative(destination, edgeFunctionsPath),
  );
  const settings = JSONC.assign(existingSettings, {
    "deno.enable": true,
    "deno.enablePaths": existingSettings["deno.enablePaths"] || [],
    "deno.unstable": true,
  });

  // If the Edge Functions path isn't already in `deno.enabledPaths`, let's add
  // it.
  if (!settings["deno.enablePaths"].includes(relativeEdgeFunctionsPath)) {
    settings["deno.enablePaths"].push(relativeEdgeFunctionsPath);
  }

  // If the Deno CLI binary isn't globally installed, we need to set the path
  // to it in the settings file or the extension won't know where to find it.
  // The only exception is when `deno.path` has already been defined, because
  // we don't want to override that.
  if (!denoBinary.global && settings["deno.path"] === undefined) {
    settings["deno.path"] = denoBinary.path;
  }

  return settings;
};

export const writeSettings = async ({
  settings,
  settingsPath,
}: {
  settings: any;
  settingsPath: string;
}) => {
  const serializedSettings = JSONC.stringify(settings, null, 2);

  await mkdir(dirname(settingsPath), { recursive: true });
  await writeFile(settingsPath, serializedSettings);
};

export const promptEditorHelper = async ({
  destination,
}: {
  destination: string;
}) => {
  const deno = new DenoBridge({
    onBeforeDownload: () =>
      console.log(
        `Setting up the Edge Functions environment. This may take a couple of minutes.`,
      ),
  });
  const denoBinary = await deno.getBinaryPath();
  const settingsPath = join(destination, ".vscode", "settings.json");
  const edgeFunctionsPath = join(destination, "src", "edge-functions");
  const { fileExists, settings: existingSettings } =
    await getSettings(settingsPath);
  const settings = applySettings(existingSettings, {
    denoBinary,
    edgeFunctionsPath,
    destination,
  });

  const createOrUpdateSettings = await confirm({
    message: fileExists
      ? `There is a VS Code settings file at ${settingsPath}. Can we update it?`
      : `A new VS Code settings file will be created at ${settingsPath}`,
    default: true,
  });

  if (!createOrUpdateSettings) {
    return;
  }

  try {
    const denoExtConfirm = await confirm({
      message:
        "The Deno VS Code extension is recommended. Would you like to install it now?",
      default: true,
    });

    if (denoExtConfirm) {
      console.log("Opening your browser...");
      open("https://ntl.fyi/deno-vscode");
    }
  } catch {
    console.log(
      "Something went wrong opening the browser, install the deno vs-code extension manually here: https://ntl.fyi/deno-vscode",
    );
  }

  try {
    await writeSettings({ settings, settingsPath });

    console.log(`VS Code settings file ${fileExists ? "updated" : "created"}.`);
  } catch {
    exitWithError("Could not write VS Code settings file.");
  }
};

export const isNetlifyCLIInstalled = () => {
  try {
    execSync("netlify --version", { stdio: "ignore" });
    return true;
  } catch {
    return false;
  }
};

export const checkConnectorBoilerplate = async (
  hasConnectorBoilerplate: boolean,
) => {
  let connectorSupportsConnect = false;
  let connectorSupportsVisualEditor = false;

  if (hasConnectorBoilerplate) {
    connectorSupportsConnect = await select<boolean>({
      message: "Will your connector support Netlify Connect?",
      choices: [
        { name: "Yes", value: true },
        { name: "No", value: false },
      ],
    });
  }

  if (hasConnectorBoilerplate) {
    connectorSupportsVisualEditor = await select<boolean>({
      message: "Will your connector support Netlify Visual Editor?",
      choices: [
        { name: "Yes", value: true },
        { name: "No", value: false },
      ],
    });
  }

  if (
    hasConnectorBoilerplate &&
    !connectorSupportsVisualEditor &&
    !connectorSupportsConnect
  ) {
    exitWithError(
      "A connector must support Netlify Connect, Netlify Visual Editor or both",
    );
  }

  return {
    visualEditor: connectorSupportsVisualEditor,
    connect: connectorSupportsConnect,
  };
};
